package org.nakedobjects.nof.reflect.remote;

import static org.easymock.EasyMock.aryEq;
import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.expectLastCall;
import static org.easymock.EasyMock.reset;
import junit.framework.TestCase;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.easymock.EasyMock;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.Oid;
import org.nakedobjects.noa.adapter.Version;
import org.nakedobjects.noa.persist.ConcurrencyException;
import org.nakedobjects.noa.security.Session;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.reflect.remote.data.ClientActionResultData;
import org.nakedobjects.nof.reflect.remote.data.ClientTransactionEvent;
import org.nakedobjects.nof.reflect.remote.data.DummyClientResultData;
import org.nakedobjects.nof.reflect.remote.data.DummyIdentityData;
import org.nakedobjects.nof.reflect.remote.data.DummyObjectData;
import org.nakedobjects.nof.reflect.remote.data.KnownObjects;
import org.nakedobjects.nof.reflect.remote.data.ObjectData;
import org.nakedobjects.nof.reflect.remote.data.ObjectEncoder;
import org.nakedobjects.nof.reflect.remote.data.ReferenceData;
import org.nakedobjects.nof.testsystem.TestProxyNakedObject;
import org.nakedobjects.nof.testsystem.TestProxySystem;
import org.nakedobjects.nof.testsystem.TestProxyVersion;
import org.nakedobjects.testing.NullSession;

public class ServerDistributionClientActionTest extends TestCase {

    public static void main(final String[] args) {
        junit.textui.TestRunner.run(ServerDistributionClientActionTest.class);
    }

    private ServerDistribution server;
    private Session session;
    private ObjectEncoder encoder;
    private TestProxySystem system;

    /*
     * Testing the Distribution implementation ServerDistribution. This uses the encoder to unmarshall objects
     * and then calls the persistor and reflector; all of which should be mocked.
     */
    protected void setUp() throws Exception {
        BasicConfigurator.configure();
        LogManager.getRootLogger().setLevel(Level.OFF);

        encoder = createMock(ObjectEncoder.class);

        server = new ServerDistribution(null);
        server.setEncoder(encoder);

        server.init();
        reset(encoder); // server calls encoder, so we need to ignore this

      // movieSpecification = new MovieSpecification();

        system = new TestProxySystem();
        system.init();

        session = NakedObjectsContext.getSession();
    }

    protected void tearDown() throws Exception {
        system.shutdown();
    }

    public void testExecuteClientActionWithNoWork() {
        NakedObjectsContext.getObjectPersistor().startTransaction();
        encoder.createClientActionResult(aryEq(new ObjectData[0]), aryEq(new Version[0]), aryEq(new ObjectData[0]));
        
        expectLastCall().andReturn(new DummyClientResultData(new ObjectData[0], new Version[0], null));
        NakedObjectsContext.getObjectPersistor().endTransaction();

        replay();
        ClientActionResultData result = server.executeClientAction(session, new ReferenceData[0], new int[0]);
        verify();

        assertEquals(0, result.getPersisted().length);
        assertEquals(0, result.getChanged().length);
    }

    public void testExecuteClientActionWhereObjectChanged() {
        NakedObject adapter = system.createPersistentTestObject();

        DummyObjectData data = new DummyObjectData(adapter.getOid(), "none", true, new TestProxyVersion(1));

        // prepare the update data to return
        expect(encoder.restore(data, new KnownObjects())).andReturn(adapter);
        

        // results returned in their own container
        DummyClientResultData results = new DummyClientResultData(new ObjectData[0], new Version[0], null);
        encoder.createClientActionResult(aryEq(new ReferenceData[1]), aryEq(new Version[] { new TestProxyVersion(2) }), aryEq(new ObjectData[0]));
//        encoderControl.setMatcher(MockControl.ARRAY_MATCHER);
        expectLastCall().andReturn(results);

        replay();
        ClientActionResultData result = server.executeClientAction(session, new ReferenceData[] { data },
                new int[] { ClientTransactionEvent.CHANGE });
        verify();

        NakedObject object = NakedObjectsContext.getObjectPersistor().getObject(adapter.getOid(), adapter.getSpecification());
        assertEquals(new TestProxyVersion(2), object.getVersion());
            
        assertEquals(result, result);
    }

    private void verify() {
        EasyMock.verify(encoder);
    }

    private void replay() {
        EasyMock.replay(encoder);
    }

    public void testExecuteClientActionWhereObjectMadePersistent() {
        NakedObject adapter = system.createTransientTestObject();

        DummyObjectData data = new DummyObjectData(adapter.getOid(), "none", true, new TestProxyVersion(1));

        // restore the object on the server
        expect(encoder.restore(data, new KnownObjects())).andReturn(adapter);
        expect(encoder.createIdentityData(adapter)).andReturn(null);

        // return results
        DummyClientResultData results = new DummyClientResultData(new ObjectData[0], new Version[0], new ObjectData[0]);
        encoder.createClientActionResult(aryEq(new ReferenceData[1]), aryEq(new Version[1]), aryEq(new ObjectData[0]));
        expectLastCall().andReturn(results);

        replay();
        ClientActionResultData result = server.executeClientAction(session, new ReferenceData[] { data },
                new int[] { ClientTransactionEvent.ADD });
        verify();

        assertEquals(result, result);
        
        NakedObject object = NakedObjectsContext.getObjectPersistor().getObject(adapter.getOid(), adapter.getSpecification());
        assertEquals(adapter, object);
        assertEquals(new TestProxyVersion(1), object.getVersion());
    }
    

    public void testExecuteClientActionFailsWithConcurrencyError() {
        NakedObject adapter = system.createPersistentTestObject();
        adapter.setOptimisticLock(new TestProxyVersion(7));
        
        Oid oid = adapter.getOid();
        DummyIdentityData identityData = new DummyIdentityData(oid, TestProxyNakedObject.class.getName(), new TestProxyVersion(6));

        try {
            server.executeClientAction(new NullSession(), new ReferenceData[] { identityData },
                    new int[] { ClientTransactionEvent.DELETE });
            fail();
        } catch (ConcurrencyException expected) {
        }
    }
    
    public void testExecuteClientActionWhereObjectDeleted() {
        NakedObject adapter = system.createPersistentTestObject();
        
        Oid oid = adapter.getOid();
        DummyIdentityData identityData = new DummyIdentityData(oid, TestProxyNakedObject.class.getName(), new TestProxyVersion(1));

        // return results
        encoder.createClientActionResult(aryEq(new ReferenceData[1]), aryEq(new Version[1]) ,aryEq(new ObjectData[0]));

        DummyClientResultData results = new DummyClientResultData(new ObjectData[0], new Version[0], null);
        expectLastCall().andReturn(results);

        replay();
        ClientActionResultData result = server.executeClientAction(new NullSession(), new ReferenceData[] { identityData },
                new int[] { ClientTransactionEvent.DELETE });
        verify();

        
        assertEquals(results, result);
    }

}
// Copyright (c) Naked Objects Group Ltd.
