package org.nakedobjects.nof.reflect.remote;

import junit.framework.TestCase;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.easymock.MockControl;
import org.nakedobjects.noa.persist.ConcurrencyException;
import org.nakedobjects.noa.reflect.NakedObjectField;
import org.nakedobjects.noa.reflect.ValueAssociation;
import org.nakedobjects.noa.security.Session;
import org.nakedobjects.nof.core.adapter.value.StringAdapter;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.reflect.Allow;
import org.nakedobjects.nof.core.reflect.Veto;
import org.nakedobjects.nof.reflect.remote.data.DummyReferenceData;
import org.nakedobjects.nof.reflect.remote.data.DummyValueData;
import org.nakedobjects.nof.reflect.remote.data.ObjectData;
import org.nakedobjects.nof.reflect.remote.data.ObjectEncoder;
import org.nakedobjects.testing.DummyNakedObject;
import org.nakedobjects.testing.DummyOid;
import org.nakedobjects.testing.DummyVersion;
import org.nakedobjects.testing.TestPersistor;
import org.nakedobjects.testing.TestSpecification;
import org.nakedobjects.testing.TestSystem;
import org.nakedobjects.testing.TestValueAdapter;



public class ServerDistributionOneToOneAssociationsTest extends TestCase {

    public static void main(final String[] args) {
        junit.textui.TestRunner.run(ServerDistributionOneToOneAssociationsTest.class);
    }

    private TestPersistor persistor;
    private ServerDistribution server;
    private Session session;
    private MockControl encoderControl;
    private ObjectEncoder encoder;
    private TestSystem system;
    private ValueAssociation association;
    private MockControl associationControl;
    private DummyNakedObject movieAdapter;
    private TestSpecification spec;
    private DummyReferenceData movieData;
    private StringAdapter stringAdapter;

    /*
     * Testing the Distribution implementation ServerDistribution.  This uses the encoder to unmarshall objects and then calls
     * the persistor and reflector; all of which should be mocked.
     */
    protected void setUp() throws Exception {
        BasicConfigurator.configure();
        LogManager.getRootLogger().setLevel(Level.OFF);
        
        encoderControl = MockControl.createControl(ObjectEncoder.class);
        encoder = (ObjectEncoder) encoderControl.getMock();

        server = new ServerDistribution(null);
        server.setEncoder(encoder);
        



        server.init();
        encoderControl.reset();  // server calls encoder, so we need to ignore this
        
        persistor = new TestPersistor();
        
        system = new TestSystem();
        system.setObjectPersistor(persistor);
        system.init();

        session = NakedObjectsContext.getSession();

        
        system.addSpecificationToLoader("none");
        
        stringAdapter = new StringAdapter();
        movieAdapter = new DummyNakedObject(new DummyOid(1, true), new DummyVersion(2));
        persistor.setupAddObject(movieAdapter);

        associationControl = MockControl.createControl(ValueAssociation.class);
        association = (ValueAssociation) associationControl.getMock();
        
        spec = new TestSpecification();
        spec.setupFields(new NakedObjectField[] {association});
        // target object has a specification & is available from persistor
        movieAdapter.setupSpecification(spec);

        movieData = new DummyReferenceData(new DummyOid(1, true), "none", new DummyVersion(2));
    }

    protected void tearDown() throws Exception {
        system.shutdown();
    }
    
    /*
     * FIX no remote clear value call; there is method on the field class though

    public void testClearAssociation() {
        /*
         * other tests for clear:
         *  - clear collection element
         *  - fails if unauthorised
         *  - fails if unavailable 
         *  
         *  could place all these clear test in one class; test other methods in other classes
         * /

        
        // the field must be authorised and available 
        associationControl.expectAndReturn(association.getId(), "director");
        associationControl.expectAndReturn(association.isAuthorised(), true);
        associationControl.expectAndReturn(association.isAvailable(movieAdapter), Allow.DEFAULT);

        // after checking association is asked to do clearing
        association.clearValue(movieAdapter);
        
        replay();
        ObjectData[] updatesData = server.clear(session, "director", movieData, personData);
        verify();
       
        assertEquals(0, updatesData.length);
    }
    */

    private void verify() {
        associationControl.verify();
        encoderControl.verify();
    }

    private void replay() {
        associationControl.replay();
        encoderControl.replay();
    }
    
    


    public void testSetValue() {
        associationControl.expectAndReturn(association.getSpecification(), new TestSpecification());
        
        // the field must be authorised and available 
        associationControl.expectAndReturn(association.getId(), "name");
        associationControl.expectAndReturn(association.isVisible(), true);
        associationControl.expectAndReturn(association.isUsable(movieAdapter), Allow.DEFAULT);

        // the value adapter is accessed and then changed
        associationControl.expectAndReturn(association.get(movieAdapter), stringAdapter);
        
        // after checking association is asked to do setting
        association.setValue(movieAdapter, new TestValueAdapter("name of movie"));
   
        replay();
        ObjectData[] updates = server.setValue(session, "name", movieData, new DummyValueData("name of movie"));
        verify();

        assertEquals("name of movie", stringAdapter.stringValue());
        assertEquals(0, updates.length);
    }

    

    
    public void testSetAssociationFailsWithNonCurrentTarget() {
        // version should be different, causing concurrency exception
        movieAdapter.setOptimisticLock(new DummyVersion(6));

        replay();
        try {
            server.setValue(session, "director", movieData, new DummyValueData("test data"));
            fail();
        } catch (ConcurrencyException expected) {}
        verify();
    }
        
    public void testSetAssociationFailsWhenUnauthorised() {
        associationControl.expectAndReturn(association.getId(), "name");
        // the field is not authorised
        associationControl.expectAndReturn(association.isVisible(), false);

        replay();
        try {
            server.setValue(session, "name", movieData, new DummyValueData("test data"));
            fail();
        } catch (IllegalRequestException expected) {
            assertEquals("can't modify field as not visible or editable", expected.getMessage());
        }
        verify();
    }

    public void testSetAssociationFailsWhenUnavailable() {
        associationControl.expectAndReturn(association.getId(), "name");
        // the field is authorised, but not available
        associationControl.expectAndReturn(association.isVisible(), true);
        associationControl.expectAndReturn(association.isUsable(movieAdapter), Veto.DEFAULT);

        replay();
        try {
            server.setValue(session, "name", movieData, new DummyValueData("test data"));
            fail();
        } catch (IllegalRequestException expected) {
            assertEquals("can't modify field as not visible or editable", expected.getMessage());
        }
        verify();
    }

    
}
// Copyright (c) Naked Objects Group Ltd.
