package org.nakedobjects.nof.reflect.remote;

import junit.framework.TestCase;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.nakedobjects.noa.adapter.Naked;
import org.nakedobjects.noa.adapter.Oid;
import org.nakedobjects.noa.reflect.NakedObjectAction;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.reflect.remote.data.DummyIdentityData;
import org.nakedobjects.nof.reflect.remote.data.DummyObjectData;
import org.nakedobjects.nof.reflect.remote.data.DummyReferenceData;
import org.nakedobjects.nof.reflect.remote.data.ObjectData;
import org.nakedobjects.nof.reflect.remote.data.ObjectEncoder;
import org.nakedobjects.nof.reflect.remote.data.ReferenceData;
import org.nakedobjects.nof.reflect.remote.data.ServerActionResultData;
import org.nakedobjects.nof.testsystem.TestProxyNakedObject;
import org.nakedobjects.nof.testsystem.TestProxySpecification;
import org.nakedobjects.nof.testsystem.TestProxySystem;
import org.nakedobjects.nof.testsystem.TestProxyVersion;
import org.nakedobjects.testing.NullSession;

import test.org.nakedobjects.objects.bom.Movie;

import static org.easymock.EasyMock.aryEq;
import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.eq;
import static org.easymock.EasyMock.expectLastCall;
import static org.easymock.EasyMock.isNull;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.reset;
import static org.easymock.EasyMock.verify;


public class ServerDistributionRemoteActionTest extends TestCase {

    public static void main(final String[] args) {
        junit.textui.TestRunner.run(ServerDistributionRemoteActionTest.class);
    }

    private ServerDistribution server;
    private ObjectEncoder encoder;
    private TestProxySystem system;
    private TestProxyNakedObject adapter;
    private Oid oid;
    private DummyIdentityData targetData;
    private ReferenceData[] parameterData;
    private NakedObjectAction action;

    /*
     * Testing the Distribution implementation ServerDistribution. This uses the encoder to unmarshall objects
     * and then calls the persistor and reflector; all of which should be mocked.
     */
    protected void setUp() throws Exception {
        BasicConfigurator.configure();
        LogManager.getRootLogger().setLevel(Level.OFF);

        encoder = createMock(ObjectEncoder.class);

        server = new ServerDistribution(null);
        server.setEncoder(encoder);

        server.init();
        reset(encoder); // server calls encoder, so we need to ignore this

        system = new TestProxySystem();
        system.init();

        adapter = system.createPersistentTestObject();
        oid = adapter.getOid();

        targetData = new DummyIdentityData(oid, TestProxyNakedObject.class.getName(), new TestProxyVersion(1));
        parameterData = new ReferenceData[] {};
        TestProxySpecification proxySpecification = (TestProxySpecification) adapter.getSpecification();
        action = createMock(NakedObjectAction.class);
        proxySpecification.setupAction(action);
        action.getId();
        expectLastCall().andStubReturn("action()");
        action.execute(eq(adapter), aryEq(new Naked[0]));
        expectLastCall().andReturn(adapter);
    }

    protected void tearDown() throws Exception {
        system.shutdown();
    }

    public void testExecuteOK() {
        ServerActionResultData results = encoder.createServerActionResult((Naked) eq(adapter), aryEq(new ObjectData[0]),
                aryEq(new ReferenceData[0]), (ObjectData) isNull(), aryEq(new ObjectData[0]), aryEq(new String[0]),
                aryEq(new String[0]));
        expectLastCall().andReturn(null);

        replay(encoder, action);

        ServerActionResultData result = server.executeServerAction(new NullSession(), NakedObjectAction.USER.getName(),
                Movie.class.getName() + "#action()", targetData, parameterData);

        verify(encoder, action);

        assertEquals(results, result);
    }

    public void testExecuteWhereObjectDeleted() {
        encoder.createIdentityData(adapter);
        ReferenceData deletedObjectIdentityData = new DummyReferenceData(adapter.getOid(), "", adapter.getVersion());
        expectLastCall().andReturn(deletedObjectIdentityData);

        ServerActionResultData results = encoder.createServerActionResult((Naked) eq(adapter), aryEq(new ObjectData[0]),
                aryEq(new ReferenceData[] { deletedObjectIdentityData }), (ObjectData) isNull(), aryEq(new ObjectData[0]),
                aryEq(new String[0]), aryEq(new String[0]));
        expectLastCall().andReturn(null);

        replay(encoder, action);
        NakedObjectsContext.getUpdateNotifer().addDisposedObject(adapter);
        ServerActionResultData result = server.executeServerAction(new NullSession(), NakedObjectAction.USER.getName(),
                Movie.class.getName() + "#action()", targetData, parameterData);
        verify(encoder, action);

        assertEquals(results, result);
    }

    public void testExecuteWhereObjectChanged() {
        encoder.createForUpdate(adapter);
        ObjectData changedObjectData = new DummyObjectData();
        expectLastCall().andReturn(changedObjectData);

        ServerActionResultData results = encoder.createServerActionResult((Naked) eq(adapter),
                aryEq(new ObjectData[] { changedObjectData }), aryEq(new ReferenceData[0]), (ObjectData) isNull(),
                aryEq(new ObjectData[0]), aryEq(new String[0]), aryEq(new String[0]));
        expectLastCall().andReturn(null);

        replay(encoder, action);
        NakedObjectsContext.getUpdateNotifer().addChangedObject(adapter);
        ServerActionResultData result = server.executeServerAction(new NullSession(), NakedObjectAction.USER.getName(),
                Movie.class.getName() + "#action()", targetData, parameterData);
        verify(encoder, action);

        assertEquals(results, result);
    }

    public void testExecuteWhereMessagesAndWarningGenerated() {
        ServerActionResultData results = encoder.createServerActionResult((Naked) eq(adapter), aryEq(new ObjectData[0]),
                aryEq(new ReferenceData[0]), (ObjectData) isNull(), aryEq(new ObjectData[0]), aryEq(new String[] { "message 1",
                        "message 2" }), aryEq(new String[] { "warning 1", "warning 2" }));
        expectLastCall().andReturn(null);

        replay(encoder, action);

        NakedObjectsContext.getMessageBroker().addMessage("message 1");
        NakedObjectsContext.getMessageBroker().addMessage("message 2");

        NakedObjectsContext.getMessageBroker().addWarning("warning 1");
        NakedObjectsContext.getMessageBroker().addWarning("warning 2");

        ServerActionResultData result = server.executeServerAction(new NullSession(), NakedObjectAction.USER.getName(),
                Movie.class.getName() + "#action()", targetData, parameterData);

        verify(encoder, action);

        assertEquals(results, result);
    }

}
// Copyright (c) Naked Objects Group Ltd.
