package test.org.nakedobjects.object.reflect.defaults;

import junit.framework.TestSuite;

import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.easymock.MockControl;
import org.nakedobjects.noa.adapter.Naked;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.persist.ObjectPersistenceException;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.context.StaticContext;
import org.nakedobjects.nof.reflect.peer.ActionPeer;
import org.nakedobjects.nof.reflect.spec.ActionImpl;
import org.nakedobjects.testing.DummyNakedObject;
import org.nakedobjects.testing.TestConfiguration;
import org.nakedobjects.testing.TestObjectLoader;
import org.nakedobjects.testing.TestPersistor;
import org.nakedobjects.testing.TestSession;
import org.nakedobjects.testing.TestSpecification;
import org.nakedobjects.testing.TestSpecificationLoader;

import test.org.nakedobjects.object.NakedObjectTestCase;


public class ActionTest extends NakedObjectTestCase {
    public static void main(final String[] args) {
        junit.textui.TestRunner.run(new TestSuite(ActionTest.class));
    }

    private ActionImpl action;
    private ActionPeer actionPeer;
    private MockControl control;

    public ActionTest(final String name) {
        super(name);
    }

    protected void setUp() throws ObjectPersistenceException {
        LogManager.getLoggerRepository().setThreshold(Level.OFF);

        NakedObjectsContext nakedObjects = StaticContext.createInstance();
        NakedObjectsContext.setConfiguration(new TestConfiguration());
        nakedObjects.setObjectPersistor(new TestPersistor());
        nakedObjects.setReflector(new TestSpecificationLoader());
        nakedObjects.setObjectLoader(new TestObjectLoader());
        nakedObjects.setSession(new TestSession());
        
        control = MockControl.createControl(ActionPeer.class);
        actionPeer = (ActionPeer) control.getMock();
        control.expectAndReturn(actionPeer.getOnType(), new TestSpecification());
        control.replay();
        
        action = new ActionImpl("reduceheadcount", actionPeer);
        
        control.reset();
    }

    protected void tearDown() throws Exception {
        control.verify();
    }

    public void testExecutePassedOnToPeer() {
        DummyNakedObject target = new DummyNakedObject();
        target.setupSpecification(new TestSpecification());
        Naked[] parameters = new Naked[2];
        
        control.expectAndReturn(actionPeer.isOnInstance(), true);
        control.expectAndReturn( actionPeer.getOnType(), new TestSpecification());
        DummyNakedObject result = new DummyNakedObject();
        control.expectAndReturn(actionPeer.execute(target, parameters), result);
         
        control.replay();
        Naked returnObject = action.execute(target, parameters);
        assertEquals(result, returnObject);
    }

    public void testHasNoReturnPassedThrough() {
        control.expectAndReturn(actionPeer.getReturnType(), null);
        control.replay();
        assertFalse(action.hasReturn());
    }

    public void testHasReturn() {
        control.expectAndReturn(actionPeer.getReturnType(), new TestSpecification());
        control.replay();
        assertTrue(action.hasReturn());
    }

    public void testNameDefaultsToActionsMethodName() {
        control.expectAndReturn(actionPeer.getName(), null);
        control.replay();
        assertEquals("Reduceheadcount", action.getName());
    }

    public void testLabel() {
        control.expectAndReturn(actionPeer.getName(), "Action Name");
        control.replay();
        assertEquals("Action Name", action.getName());
    }

    public void testId() {
        control.replay();
        assertEquals("reduceheadcount", action.getId());
    }

    public void testReturnTypePassedThrough() {
        TestSpecification returnType = new TestSpecification();
        control.expectAndReturn(actionPeer.getReturnType(), returnType);
        control.replay();
        assertEquals(returnType, action.getReturnType());
    }

    public void testCompleteParameterNameSetPassedThrough() {
        control.expectAndReturn(actionPeer.getParameterNames(), new String[] {"one", "two", "three"});
        TestSpecification type = new TestSpecification();
        control.expectAndReturn(actionPeer.getParameterTypes(), new NakedObjectSpecification[] {type, type, type});
          
        control.replay();
        String[] parameterNames = action.getParameterNames();
        assertEquals("one", parameterNames[0]);
        assertEquals("two", parameterNames[1]);
        assertEquals("three", parameterNames[2]);
    }
    
    public void testNullParameterNamesReplacedWithTypeNames() {
        TestSpecification param1Type = new TestSpecification();
        TestSpecification param2Type = new TestSpecification("two");
        TestSpecification param3Type = new TestSpecification();        
        
        control.expectAndReturn(actionPeer.getParameterNames(), new String[] {"one", null, "three"});
        control.expectAndReturn(actionPeer.getParameterTypes(), new NakedObjectSpecification[] {param1Type, param2Type, param3Type});
          
        control.replay();
        String[] parameterNames = action.getParameterNames();
        assertEquals("one", parameterNames[0]);
        assertEquals("two (singular)", parameterNames[1]);
        assertEquals("three", parameterNames[2]);
    }

    public void testNullParameterNameSetReplacedBySetOfNamesFromTypes() {
        TestSpecification param1Type = new TestSpecification("one");
        TestSpecification param2Type = new TestSpecification("two");
        TestSpecification param3Type = new TestSpecification("three");        
        
        control.expectAndReturn(actionPeer.getParameterNames(), null);
        control.expectAndReturn(actionPeer.getParameterTypes(), new NakedObjectSpecification[] {param1Type, param2Type, param3Type});
          
        control.replay();
        String[] parameterNames = action.getParameterNames();
        assertEquals("one (singular)", parameterNames[0]);
        assertEquals("two (singular)", parameterNames[1]);
        assertEquals("three (singular)", parameterNames[2]);
    }
    
    public void testDefaultParameterValues() {
        DummyNakedObject target = new DummyNakedObject();
        target.setupSpecification(new TestSpecification());
        control.expectAndReturn( actionPeer.isOnInstance(), true);
        control.expectAndReturn( actionPeer.getOnType(), new TestSpecification());
        control.expectAndReturn( actionPeer.getOnType(), new TestSpecification());
        control.expectAndReturn(actionPeer.getParameterCount(), 0);
        control.expectAndReturn(actionPeer.getParameterTypes(), new NakedObjectSpecification[0]);
        control.expectAndReturn(actionPeer.getParameterDefaults(target), new Object[0]);

        control.replay();
        Naked[] defaults = action.getDefaultParameterValues(target);
    }
    
}
// Copyright (c) Naked Objects Group Ltd.
